import {
  AutocompletePicklistItem
} from '../interface/autocomplete-item';
import { Component, Input, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { Observable } from 'rxjs';
import { map, startWith } from 'rxjs/operators';


/** A picklist with typeahead filtering. */
@Component({
  selector: 'sis-generic-typeahead-picklist',
  styleUrls: ['./generic-typeahead.component.scss'],
  templateUrl: './generic-typeahead.component.html',
})
export class GenericTypeaheadComponent implements OnInit {

  // Allow values to be added even if they aren't in the provided list.
  @Input() allowValuesNotInPicklist: boolean;

  // (Accessibility) Name of the field for screen readers.
  @Input() ariaLabel: string;

  // List of values displayed in the control. This is a subset of the
  // picklistItems that is filtered by the text in the input box.
  filteredItems: Observable<AutocompletePicklistItem[]>;

  // Name of the form group field that is tied to this control.
  @Input() formFieldName: string;

  // The form group (see Angular reactive forms) associated with this control.
  @Input() formGroup: FormGroup;

  // Placeholder text when the input field is null.
  @Input() placeholderText: string;

  // List of available options that the user can choose from.
  @Input() picklistItems: Map<string, AutocompletePicklistItem>;

  selectedItem: any;

  constructor() {
    // No action.
  }

  /** Filter list choices based on text entered in the input box.
   * @param value Form group selected value.
   * @returns List of filtered items.
   */
  protected filter(value: any): AutocompletePicklistItem[] {
    const text = this.getInputText(value);
    if (!text) {
      return Array.from(this.picklistItems.values());
    }
    const filterValue = text.toLowerCase();
    return Array.from(this.picklistItems.values()).filter(
      (item: AutocompletePicklistItem): boolean => {
        if (item.filterCustom) {
          return item.filterCustom(filterValue);
        }
        return item.getName().toLowerCase().includes(filterValue);
      }
    );
  }

  /** Get the text of the input field.
   * @param value The value changes of the form control.
   * @returns Text of the input field.
   */
  protected getInputText(value: any): string {
    const key = this.formFieldName;
    if (value) {
      if (value.getName) {
        return value.getName();
      }
      if (value.hasOwnProperty(key)) {
        if (value[key].getName) {
          return value[key].getName();
        }
        if (typeof value[key] === 'string') {
          return value[key];
        }
      }
      if (typeof value === 'string') {
        return value;
      }
      console.log(`Unrecognized input text value: ${value}`);
    }
    return null;
  }

  /** Return the text that is displayed by the control.
   * @param value Data representation.
   * @returns Text to show.
   */
  getText(value: AutocompletePicklistItem | string): string {
    if (typeof value === 'string') {
      return value;
    }
    return value.getName();
  }

  /** Setup the component. */
  ngOnInit() {
    this.filteredItems = this.formGroup.valueChanges
      .pipe(
        startWith(''),
        map((value: any): AutocompletePicklistItem[] => {
          return this.filter(value);
        })
      );
  }

  /** Handle loss of focus on the control.
   * Force user to choose one of the available options.
   * See https://stackoverflow.com/a/51427590/241025
   */
  onBlur(): void {
    // TODO see if you can pass the event here and stop it in the right scenario
    if (!this.allowValuesNotInPicklist) {
      const key = this.formFieldName;
      if (
        !this.selectedItem ||
        this.selectedItem !== this.formGroup.controls[key].value
      ) {
        // TODO check to make sure that changing the form group value
        //      doesn't trigger the event
        this.formGroup.controls[key].setValue('');
        this.selectedItem = '';
      }
    }
  }

  /** One of of the options from the dropdown list was selected.
   * @param event The angular event that contains the data.
   */
  onOptionSelected(event: any): void {
    if (!this.allowValuesNotInPicklist) {
      this.selectedItem = event.option.value;
    }
  }

}
